;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "lsp-ivy" "20260507.1752"
  "LSP ivy integration."
  '((emacs    "29.1")
    (dash     "2.14.1")
    (lsp-mode "6.2.1")
    (ivy      "0.13.0"))
  :url "https://github.com/emacs-lsp/lsp-ivy"
  :commit "c0930544948dfdb7bf497fc9e58aa6b4b857e237"
  :revdesc "c0930544948d"
  :keywords '("languages" "debug"))
